module Test = Integration_test_base

let foo_name = "\\Foo"

let bar_name = "\\Bar"

let foo_file_name = "foo.php"

let bar_file_name = "bar.php"

let foo_contents =
  {|<?hh //strict

const string GLOBAL_X = "X";

class Foo {
  // this is an error (lack of typehint) revealed during declaration only
  const X = GLOBAL_X;
}
|}

let bar_contents =
  {|<?hh //strict
class Bar {
  public function test(Foo $x) : void {}
}
|}

let expected_errors =
  {|
File "/foo.php", line 7, characters 9-9:
Please add a type hint (Naming[2035])
|}

let test () =
  let env = Test.setup_server () in
  let env =
    Test.setup_disk
      env
      [(foo_file_name, foo_contents); (bar_file_name, bar_contents)]
  in
  let ctx = Provider_utils.ctx_from_server_env env in
  let classes = [foo_name; bar_name] in
  let foo_path = Relative_path.from_root ~suffix:foo_file_name in
  let bar_path = Relative_path.from_root ~suffix:bar_file_name in
  (* Remove things from shared memory (that were put there by Test.setup_disk)
   * to simulate lazy saved state init. *)
  let defs =
    { FileInfo.empty_names with FileInfo.n_classes = SSet.of_list classes }
  in
  let elems = Decl_class_elements.get_for_classes ~old:false classes in
  Decl_redecl_service.remove_defs ctx ~collect_garbage:false defs elems;
  SharedMem.invalidate_local_caches ();

  (* Local caches need to be invalidated whenever things are removed from shared
   * memory (to avoid getting cached old versions of declarations) *)
  let empty = Relative_path.Set.empty in
  let memory_cap = None in
  let check_info =
    {
      Typing_service_types.init_id = "";
      recheck_id = Some "";
      profile_log = false;
      profile_type_check_twice = false;
      profile_type_check_duration_threshold = 0.;
      profile_type_check_memory_threshold_mb = 0;
      profile_decling = Typing_service_types.DeclingOff;
    }
  in
  let delegate_state = Typing_service_delegate.default in
  let { Typing_check_service.errors; delegate_state; telemetry; _ } =
    Typing_check_service.go
      ctx
      None
      delegate_state
      (Telemetry.create ())
      empty
      [bar_path]
      ~memory_cap
      ~longlived_workers:false
      ~remote_execution:None
      ~check_info
      ~cgroup_step:None
  in
  Test.assert_errors errors "";
  let { Typing_check_service.errors; delegate_state; telemetry; _ } =
    Typing_check_service.go
      ctx
      None
      delegate_state
      telemetry
      empty
      [bar_path]
      ~memory_cap
      ~longlived_workers:false
      ~remote_execution:None
      ~check_info
      ~cgroup_step:None
  in
  Test.assert_errors errors "";

  let { Typing_check_service.errors; delegate_state; telemetry; _ } =
    Typing_check_service.go
      ctx
      None
      delegate_state
      telemetry
      empty
      [foo_path]
      ~memory_cap
      ~longlived_workers:false
      ~remote_execution:None
      ~check_info
      ~cgroup_step:None
  in
  Test.assert_errors errors expected_errors;
  let { Typing_check_service.errors; _ } =
    Typing_check_service.go
      ctx
      None
      delegate_state
      telemetry
      empty
      [foo_path]
      ~memory_cap
      ~longlived_workers:false
      ~remote_execution:None
      ~check_info
      ~cgroup_step:None
  in
  Test.assert_errors errors expected_errors;

  ignore env;
  ()
